import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';

import '../../../../../utils/node_utils.dart';
import '../../../models/internal_resolved_unit_result.dart';
import '../../../models/issue.dart';
import '../../../models/severity.dart';
import '../../flutter_rule_utils.dart';
import '../../models/flutter_rule.dart';
import '../../models/rule_documentation.dart';
import '../../rule_utils.dart';

part 'config_parser.dart';
part 'visitor.dart';

class PreferSingleWidgetPerFileRule extends FlutterRule {
  static const String ruleId = 'prefer-single-widget-per-file';

  static const _warningMessage = 'Only a single widget per file is allowed.';

  final bool _ignorePrivateWidgets;

  PreferSingleWidgetPerFileRule([Map<String, Object> config = const {}])
      : _ignorePrivateWidgets = _ConfigParser.parseIgnorePrivateWidgets(config),
        super(
          id: ruleId,
          documentation: const RuleDocumentation(
            name: 'Prefer a single widget per file',
            brief: 'Warns when a file contains more than a single widget.',
          ),
          severity: readSeverity(config, Severity.style),
          excludes: readExcludes(config),
        );

  @override
  Iterable<Issue> check(InternalResolvedUnitResult source) {
    final visitor = _Visitor(ignorePrivateWidgets: _ignorePrivateWidgets);

    source.unit.visitChildren(visitor);

    return visitor.nodes
        .map(
          (node) => createIssue(
            rule: this,
            location: nodeLocation(
              node: node,
              source: source,
            ),
            message: _warningMessage,
          ),
        )
        .toList(growable: false);
  }
}
